'use strict';
var striptags = require('striptags');

const white = new RGB(255, 255, 255);
const lightgrey = new RGB(190, 190, 190);

function toHex(n) {
	var hex = Number(n).toString(16);
	if(hex.length < 2) {
		hex = '0' + hex;
	}
	return hex;
}

function formatMessage(color, message) {
	var hexColor = toHex(color.r) + toHex(color.g) + toHex(color.b);
	return '<font style=\"color: #' + hexColor + '\">' + message + '</font>';
}

const commandManager = bullymp.events.call('get_command_manager')[0];

function onPlayerMessage(player, message) {
    if(message.length < 1)
        return;

	message = striptags(message, ['s', 'u']); // myu: make client messages html safe, but give them some options atleast.

	if(message.startsWith('/')) {
		let command = message.substr(1);
		let args = command.match(/('(\\'|[^'])*'|"(\\"|[^"])*"|\/(\\\/|[^\/])*\/|(\\ |[^ ])+|[\w-]+)/g) || [];
		for(var i = 1; i < args.length; i++) {
			if(args[i].substr(0, 1) === '"' || args[i].substr(0, 1) === "'") {
				args[i] = JSON.parse(args[i]);
			}
		}
		if(args.length === 0) {
			return;
		}
		let commandName = args.splice(0, 1)[0];
		if(!commandManager.handle(player, commandName, args)) {
			chat.send(player, lightgrey, "Unknown command!");
		}
	} 
	else {
		if(!bullymp.events.call('chat.onPlayerMessage', player, message)[0])
			return;
		chat.broadcast(white, formatMessage(player.color, player.name + ': ') + message);
	}
}

bullymp.events.addRemoteCallable('chat.sendMessage', onPlayerMessage);

const chat = {
	colors: {
		maroon: new RGB(128,0,0),
		darkred: new RGB(139,0,0),
		brown: new RGB(165,42,42),
		firebrick: new RGB(178,34,34),
		crimson: new RGB(220,20,60),
		red: new RGB(255,0,0),
		tomato: new RGB(255,99,71),
		coral: new RGB(255,127,80),
		indianred: new RGB(205,92,92),
		lightcoral: new RGB(240,128,128),
		darksalmon: new RGB(233,150,122),
		salmon: new RGB(250,128,114),
		lightsalmon: new RGB(255,160,122),
		orangered: new RGB(255,69,0),
		darkorange: new RGB(255,140,0),
		orange: new RGB(255,165,0),
		gold: new RGB(255,215,0),
		darkgoldenrod: new RGB(184,134,11),
		goldenrod: new RGB(218,165,32),
		palegoldenrod: new RGB(238,232,170),
		darkkhaki: new RGB(189,183,107),
		khaki: new RGB(240,230,140),
		olive: new RGB(128,128,0),
		yellow: new RGB(255,255,0),
		yellowgreen: new RGB(154,205,50),
		darkolivegreen: new RGB(85,107,47),
		olivedrab: new RGB(107,142,35),
		lawngreen: new RGB(124,252,0),
		chartreuse: new RGB(127,255,0),
		greenyellow: new RGB(173,255,47),
		darkgreen: new RGB(0,100,0),
		green: new RGB(0,128,0),
		forestgreen: new RGB(34,139,34),
		lime: new RGB(0,255,0),
		limegreen: new RGB(50,205,50),
		lightgreen: new RGB(144,238,144),
		palegreen: new RGB(152,251,152),
		darkseagreen: new RGB(143,188,143),
		mediumspringgreen: new RGB(0,250,154),
		springgreen: new RGB(0,255,127),
		seagreen: new RGB(46,139,87),
		mediumaquamarine: new RGB(102,205,170),
		mediumseagreen: new RGB(60,179,113),
		lightseagreen: new RGB(32,178,170),
		darkslategray: new RGB(47,79,79),
		teal: new RGB(0,128,128),
		darkcyan: new RGB(0,139,139),
		aqua: new RGB(0,255,255),
		cyan: new RGB(0,255,255),
		lightcyan: new RGB(224,255,255),
		darkturquoise: new RGB(0,206,209),
		turquoise: new RGB(64,224,208),
		mediumturquoise: new RGB(72,209,204),
		paleturquoise: new RGB(175,238,238),
		aquamarine: new RGB(127,255,212),
		powderblue: new RGB(176,224,230),
		cadetblue: new RGB(95,158,160),
		steelblue: new RGB(70,130,180),
		cornflowerblue: new RGB(100,149,237),
		deepskyblue: new RGB(0,191,255),
		dodgerblue: new RGB(30,144,255),
		lightblue: new RGB(173,216,230),
		skyblue: new RGB(135,206,235),
		lightskyblue: new RGB(135,206,250),
		midnightblue: new RGB(25,25,112),
		navy: new RGB(0,0,128),
		darkblue: new RGB(0,0,139),
		mediumblue: new RGB(0,0,205),
		blue: new RGB(0,0,255),
		royalblue: new RGB(65,105,225),
		blueviolet: new RGB(138,43,226),
		indigo: new RGB(75,0,130),
		darkslateblue: new RGB(72,61,139),
		slateblue: new RGB(106,90,205),
		mediumslateblue: new RGB(123,104,238),
		mediumpurple: new RGB(147,112,219),
		darkmagenta: new RGB(139,0,139),
		darkviolet: new RGB(148,0,211),
		darkorchid: new RGB(153,50,204),
		mediumorchid: new RGB(186,85,211),
		purple: new RGB(128,0,128),
		thistle: new RGB(216,191,216),
		plum: new RGB(221,160,221),
		violet: new RGB(238,130,238),
		magenta: new RGB(255,0,255),
		orchid: new RGB(218,112,214),
		mediumvioletred: new RGB(199,21,133),
		palevioletred: new RGB(219,112,147),
		deeppink: new RGB(255,20,147),
		hotpink: new RGB(255,105,180),
		lightpink: new RGB(255,182,193),
		pink: new RGB(255,192,203),
		antiquewhite: new RGB(250,235,215),
		beige: new RGB(245,245,220),
		bisque: new RGB(255,228,196),
		blanchedalmond: new RGB(255,235,205),
		wheat: new RGB(245,222,179),
		cornsilk: new RGB(255,248,220),
		lemonchiffon: new RGB(255,250,205),
		lightgoldenrodyellow: new RGB(250,250,210),
		lightyellow: new RGB(255,255,224),
		saddlebrown: new RGB(139,69,19),
		sienna: new RGB(160,82,45),
		chocolate: new RGB(210,105,30),
		peru: new RGB(205,133,63),
		sandybrown: new RGB(244,164,96),
		burlywood: new RGB(222,184,135),
		tan: new RGB(210,180,140),
		rosybrown: new RGB(188,143,143),
		moccasin: new RGB(255,228,181),
		navajowhite: new RGB(255,222,173),
		peachpuff: new RGB(255,218,185),
		mistyrose: new RGB(255,228,225),
		lavenderblush: new RGB(255,240,245),
		linen: new RGB(250,240,230),
		oldlace: new RGB(253,245,230),
		papayawhip: new RGB(255,239,213),
		seashell: new RGB(255,245,238),
		mintcream: new RGB(245,255,250),
		slategray: new RGB(112,128,144),
		lightslategray: new RGB(119,136,153),
		lightsteelblue: new RGB(176,196,222),
		lavender: new RGB(230,230,250),
		floralwhite: new RGB(255,250,240),
		aliceblue: new RGB(240,248,255),
		ghostwhite: new RGB(248,248,255),
		honeydew: new RGB(240,255,240),
		ivory: new RGB(255,255,240),
		azure: new RGB(240,255,255),
		snow: new RGB(255,250,250),
		black: new RGB(0,0,0),
		dimgrey: new RGB(105,105,105),
		grey: new RGB(128,128,128),
		darkgrey: new RGB(169,169,169),
		silver: new RGB(192,192,192),
		lightgrey: new RGB(211,211,211),
		gainsboro: new RGB(220,220,220),
		whitesmoke: new RGB(245,245,245),
		white: new RGB(255,255,255)
	},

	setVisible(target, visible) {
		bullymp.events.callRemote(target, 'chat.setVisible', visible);
	},

	send(target, color, message) {
		bullymp.events.callRemote(target, 'chat.postmessage', formatMessage(color, message));
	},
	
	broadcast(color, message) {
		bullymp.events.callRemote(null, 'chat.postmessage', formatMessage(color, message));
	}
}

bullymp.events.add('get_chat', () => chat);
