'use strict';

var Timer = require('../utils/timer');

module.exports = class VehicleRespawner {
	static respawn(spawnId) {
		var vehicle = bullymp.vehicles[data.vehicleSpawns[spawnId].id];
		if(!!vehicle) {
			vehicle.destroy();
		}
		var spawnInfo = data.vehicleSpawns[spawnId];
		vehicle = new Vehicle(spawnInfo.modelId, spawnInfo.position, spawnInfo.heading, spawnInfo.color1, spawnInfo.color2);
		data.vehicleSpawns[spawnId].id = (!!vehicle ? vehicle.id : null);
	}

	static checkForObstacles(forVehicleId, checkedPosition) {
		var forVehicle = bullymp.vehicles[forVehicleId];

		if(!!forVehicle) {
			for(let i = 0; i < bullymp.players.length; ++i) {
				var player = bullymp.players[i];

				if(!!player) {
					if(gm.config.vehicle_respawn.player_free_radius > gm.utility.distanceBetween3D(checkedPosition, player.position)) {
						return true;
					}
				}
			}

			for(let i = 0; i < bullymp.vehicles.length; ++i) {
				if(i == forVehicleId) {
					continue;
				}

				var vehicle = bullymp.vehicles[i];

				if(!!vehicle) {
					if(gm.config.vehicle_respawn.vehicle_free_radius > gm.utility.distanceBetween3D(checkedPosition, vehicle.position)) {
						return true;
					}
				}
			}
		}

		return false;
	}

	static run() {
		var vehicleSpawns = data.vehicleSpawns;
		var now = new Date();

		for(let i = 0; i < vehicleSpawns.length; ++i) {
			var vehicleSpawn = vehicleSpawns[i];

			var vehicleId = vehicleSpawn.id;
			var vehicle = bullymp.vehicles[vehicleId];

			if(!!vehicle)
			{
				var vehicleIsOccupied = !!vehicle.driver;

				if(!!vehicleSpawn.respawnInfo)
				{
					if(vehicleSpawn.respawnInfo.occupied != vehicleIsOccupied) {
						vehicleSpawn.respawnInfo.lastOccupied = now;
						vehicleSpawn.respawnInfo.occupied = vehicleIsOccupied;
					}

					if(vehicleSpawn.respawnInfo.occupied == false && now - vehicleSpawn.respawnInfo.lastOccupied >= gm.config.vehicle_respawn.respawn_interval) {
						var spawnPosition = vehicleSpawn.position;
						var currentPosition = vehicle.position;

						var distance = gm.utility.distanceBetween3D(spawnPosition, currentPosition);

						if(distance >= gm.config.vehicle_respawn.required_distance && gm.vehicleRespawner.checkForObstacles(vehicleId, spawnPosition) == false) {
							gm.vehicleRespawner.respawn(i);
							//console.log(vehicleId + ' has been respawned! (distance: ' + distance + ')');

							vehicleSpawn.respawnInfo.lastOccupied = now;
						}
					}
				}
				else
				{
					vehicleSpawn.respawnInfo = {
						occupied: false,
						lastOccupied: now,
					};
				}
			}
		}
	}

	static init() {
		var timer = new Timer(function() { gm.vehicleRespawner.run(); }, gm.config.vehicle_respawn.timer_interval);
		timer.run();
	}
}
